## 正则表达式

---

在Sun的Java JDK 1.40版本中，Java自带了支持正则表达式的包，主要是放在java.util.regex包下面。

```
 Pattern p_script = Pattern.compile("正则表达式", Pattern.CASE_INSENSITIVE);
 Matcher m_script = p_script.matcher(content);
 while (m_script.find()) {
           // 找到匹配内容，进行后续事情
            String strAid = m_script.group(1);
            。。。。
 }
```
**Matcher类的常用方法：**

* matches()：返回整个目标字符串与Pattern是否匹配

* find()：返回与Pattern匹配的下一个子串

* group()：返回上一次与Pattern匹配的子串中的内容。group是针对（）来说的，group（0）就是指的整个串，group（1） 指的是第一个括号里的东西，group（2）指的第二个括号里的东西

* start()：返回上一次与Pattern匹配的子串在目标字符串中的开始位置。

* end()：返回上一次与Pattern匹配的子串在目标字符串中的结束位置加1。

**正则表达式语法**

|元字符|描述|
|--|--|
| \ | 将下一个字符标记符、或一个向后引用、或一个八进制转义符。例如，“\\n”匹配\n。“\n”匹配换行符。序列“\\”匹配“\”而“\(”则匹配“(”。即相当于多种编程语言中都有的“转义字符”的概念。|
|^ | 匹配输入字符串的开始位置。如果设置了RegExp对象的Multiline属性，^也匹配“\n”或“\r”之后的位置。|
|$ | 匹配输入字符串的结束位置。如果设置了RegExp对象的Multiline属性，$也匹配“\n”或“\r”之前的位置。|
| * | 匹配前面的子表达式任意次。例如，zo*能匹配“z”，“zo”以及“zoo”。*等价于{0,}|
|+ |匹配前面的子表达式一次或多次(大于等于1次）。例如，“zo+”能匹配“zo”以及“zoo”，但不能匹配“z”。+等价于{1,}。|
|?|匹配前面的子表达式零次或一次。例如，“do(es)?”可以匹配“do”或“does”中的“do”。?等价于{0,1}。|
|{n} |n是一个非负整数。匹配确定的n次。例如，“o{2}”不能匹配“Bob”中的“o”，但是能匹配“food”中的两个o。
|{n,} |n是一个非负整数。至少匹配n次。例如，“o{2,}”不能匹配“Bob”中的“o”，但能匹配“foooood”中的所有o。“o{1,}”等价于“o+”。“o{0,}”则等价于“o*”。|
|{n,m} |m和n均为非负整数，其中n<=m。最少匹配n次且最多匹配m次。例如，“o{1,3}”将匹配“fooooood”中的前三个o。“o{0,1}”等价于“o?”。请注意在逗号和两个数之间不能有空格。|
| x\|y |匹配x或y。例如，“z|food”能匹配“z”或“food”或"zood"(此处请谨慎)。“(z|f)ood”则匹配“zood”或“food”。|
| [xyz] |字符集合。匹配所包含的任意一个字符。例如，“[abc]”可以匹配“plain”中的“a”。
| [^xyz]  |负值字符集合。匹配未包含的任意字符。例如，“[^abc]”可以匹配“plain”中的“plin”。
| [a-z] | 字符范围。匹配指定范围内的任意字符。例如，“[a-z]”可以匹配“a”到“z”范围内的任意小写字母字符。|
| [^a-z]  | 负值字符范围。匹配任何不在指定范围内的任意字符。例如，“[^a-z]”可以匹配任何不在“a”到“z”范围内的任意字符。|
| . | 可以匹配任何字符|
| \d | 匹配一个数字字符。等价于[0-9] |
| \D | 匹配一个非数字字符。等价于[^0-9] |
| \s | 匹配所有的空白字符，包括空格、制表符、换页符、换行符、回车符 等等。等价于[ \f\n\r\t\v]。 |
| \S |匹配所有的非空白字符|

更多内容参考：http://www.cnblogs.com/lzq198754/p/5780340.html

**常用正则表达式**

|规则	|正则表达式语法  |
|--|--|
|一个或多个汉字|	^[\u0391-\uFFE5]+$ 
|邮政编码	 | ^[1-9]\d{5}$ 
|QQ号码	 | ^[1-9]\d{4,10}$ 
|用户名（字母开头 + 数字/字母/下划线）|	^[A-Za-z][A-Za-z1-9_-]+$
|手机号码	 | ^1[3|4|5|8][0-9]\d{8}$ 
|URL|	^((http|https)://)?([\w-]+\.)+[\w-]+(/[\w-./?%&=]*)?$ 
|18位身份证号 |	^(\d{6})(18|19|20)?(\d{2})([01]\d)([0123]\d)(\d{3})(\d|X|x)?$
|邮箱| ^[a-zA-Z_]{1,}[0-9]{0,}@(([a-zA-z0-9]-*){1,}\.){1,3}[a-zA-z\-]{1,}$ 



### 示例

```
示例：
\\[img(\\=(\\d+)\\,(\\d+))?\\](((http\\:|https\\:)?\\/\\/[a-zA-Z|\\.|0-9]*\\/)?[a-zA-Z|\\/|0-9]*\\.[a-zA-Z]*)\\[\\/img\\]

[img=225,165]http://bbsimg.wacdn.com/forum/201702/07/120732h4p5yw6wk4bwyk5q.jpg[/img]火炉上一分钟如一小时那


group（0）=[img=225,165]http://bbsimg.wacdn.com/forum/201702/07/120732h4p5yw6wk4bwyk5q.jpg[/img]
group（1）==225,165
group（2）=225
group（3）=165
group（4）=http://bbsimg.wacdn.com/forum/201702/07/120732h4p5yw6wk4bwyk5q.jpg
group（5）=http://bbsimg.wacdn.com/
group（6）=http:

```
